import os
import ctypes
from ctypes import *
from ctypes.util import find_library
from blc_array import BlcArray

class BlcChannel(BlcArray):
    """
        blc_channel wrapper.
    """
    # Bind blc_channel methods
    lib_path = find_library("blc")
    lib = CDLL(lib_path)
    lib.blc_channel_new.restype = c_void_p
    lib.blc_channel_new_open.restype = c_void_p
    
    lib.blc_channel_delete.argtypes = [ c_void_p ]

    lib.blc_channel_wait_new_data.argtypes = [ c_void_p ]
    lib.blc_channel_wait_ack_data.argtypes = [ c_void_p ] 
    lib.blc_channel_post_new_data.argtypes = [ c_void_p ]
    lib.blc_channel_post_ack_data.argtypes = [ c_void_p ]
    lib.blc_channel_remove.argtypes = [ c_void_p ]
    
    types={ 'UIN8': c_uint8, 'INT8': c_int8, 'UI16': c_uint16, 'INT16': c_int16, 'UI32': c_uint32, 'IN32': c_int32, 'UI64': c_uint64, 'IN64': c_int64, 'FL32': c_float, 'FL64': c_double}

    def __init__(
        self, name, mode, data_type=None,format_type=None, dims=None
    ):
        """
            Creates or reopens a given blc_channel.
            :param name: Name of the blc_channel (with sync/async identifier). Example: "/data"
            :param mode: RW mode. Example: os.O_RDWR

            TODO """
                
        if (dims): #Creates
            dim_types = [c_size_t] * len(dims)
            BlcChannel.lib.blc_channel_new.argtypes = [ c_char_p, c_int, c_uint32, c_uint32, c_int, *dim_types ]
            type_int =   int.from_bytes(data_type.encode(), "big")
            format_int = int.from_bytes(format_type.encode(), "big")

            # Create a new channel
            self.pointer = BlcChannel.lib.blc_channel_new(name.encode(), mode, type_int, format_int, len(dims), *dims)
    
        else: #Open 
            BlcChannel.lib.blc_channel_new_open.argtypes = [ c_char_p, c_int ]
            self.pointer = BlcChannel.lib.blc_channel_new_open( name.encode(), mode )
            
        self.get_def()

    
    def wait_new_data(self):
        BlcChannel.lib.blc_channel_wait_new_data(self.pointer)
    
    def wait_ack_data(self):
        BlcChannel.lib.blc_channel_wait_ack_data(self.pointer)

    def post_new_data(self):
        BlcChannel.lib.blc_channel_post_new_data(self.pointer)

    def post_ack_data(self):
        BlcChannel.lib.blc_channel_post_ack_data(self.pointer)
    
    def remove(self): 
        BlcChannel.lib.blc_channel_remove(self.pointer)
    
    def __del__(self):
        BlcChannel.lib.blc_channel_delete(self.pointer) #Free the local object but do not destroy the system shared memory
    

    @property
    def list(self):
        #cast it from
        data_list = list(self.data[:self.total_length])
        return data_list

if __name__ == "__main__":
    print()
    print("EXEMPLE: blc_channel")
    print("We create a blc_channel of type unsigned char and size 3x2 on the POSIX system (shared memory)")
    channel = BlcChannel("/python_test", os.O_RDWR, 'UIN8', 'RGB3', [3, 2])
    print("type",channel.type, "format", channel.format, "length", channel.total_length)
    
    print("We write 7 in column 2")
    channel.data[2]=7
    print("We open the shared blc_channel to read it")
    channel_read = BlcChannel("/python_test", os.O_RDONLY)
    print("We check that we have the same values")
    print("type",channel_read.type, "format", channel_read.format, "length", channel_read.total_length)
    print(channel_read.data[:channel.total_length])
    print("We remove our shared memory from the system")
    channel.remove()
    
